package com.streambase.contrib;

import java.util.ArrayList;
import java.util.List;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.client.CustomFunctionResolver;
import com.streambase.sb.operator.AggregateWindow;

public class addAggregate extends AggregateWindow {

	public static final long serialVersionUID = 1271621215860L;
	public static List<Double> sumFactors; 

	/**
	 * Called before each new use of the aggregate window object.
	 */
	public void init() {
		sumFactors = null;
	}

	/**
	 * The calculate method will be called whenever a new tuple is generated.
	 * The calculate method may be called just once (in the case when a tuple is
	 * generated when the aggregate window closes) or many times (in the case when
	 * the aggregate window remains open, but a tuple is generated because "emit every 
	 * N tuples" was selected). It is possible that calculate may be called before
	 * accumulate is called. It is also possible that calculate may never be called.  
	 */
	public List<Double> calculate() {
		return sumFactors;	
	}

	/**
	 * The accumulate method will be called each time new data arrives for the window.
	 * There may be many overloaded accumulate methods. calljava determines which one
	 * to call based on type. All accumulate methods must have a void return. There must be
	 * at least one accumulate method.
	 * The function can be assigned an alias. It can then be called using the alias name instead of
	 * using calljava().
	 */
	@CustomFunctionResolver("accumulateCustomFunctionResolver0")
	public void accumulate(List<Double> factors) {
		if (sumFactors == null)
			sumFactors = factors;
		else 
			sumFactors = addElements(sumFactors, factors);
	}

	/**
	 * 
	 * @param vec1 the vector to add to
	 * @param vec2 the vector to add from
	 * @return a sum of the two vectors
	 */
	public static List<Double> addElements(List<Double> vec1, List<Double> vec2){
		List<Double> result = new ArrayList<Double>();
		for (int i = 0; i < vec1.size(); i++){
			result.add(vec1.get(i) + vec2.get(i));
		}
		return result;
	}
	/**
	 * A StreamBase Custom Function Resolver Function. This method is used by 
	 * StreamBase for type checking.  Use this method
	 * to resolve functions that take lists or tuples as arguments 
	 * or return lists or tuples.  This method should be used to ensure the 
	 * data types of your function are correct.
	
	 * For custom functions that return simple types (the Java equivalents of the simple 
	 * StreamBase types: int, long, double, bool, string, blob, timestamp), the returned CompleteDataType
	 * must be the appropriate corresponding simple type (e.g., as returned by CompleteDataType.forInt()).
	 * For custom functions that return non-simple types (the Java equivalents of the non-simple
	 * StreamBase types: tuple, list), the returned CompleteDataType must fully describe the appropriate
	 * non-simple type (as returned by CompleteDataType.forTuple(Schema) or CompleteDataType.forList(CompleteDataType))
	 *{@see com.streambase.sb.client.CustomFunctionResolver}  
	 */
	public static CompleteDataType accumulateCustomFunctionResolver0( CompleteDataType factors) {
		if (factors.getElementType() == CompleteDataType.forDouble())
			return CompleteDataType.forList(CompleteDataType.forDouble());
		return null;
	}

	/**
	 * Called after the end of a use of the aggregate window object.
	 */
	public void release() {
		sumFactors = null;
	}

}
