package com.streambase.contrib;

import java.util.ArrayList;
import java.util.List;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.client.CustomFunctionResolver;

public class limits {

	/**
	 * 
	 * @param portfolio the risk values of the portfolio
	 * @param limits a vector of upper limits on portfolio risk
	 * @return true if all the portfolio factors are less than the risk limits, false otherwise
	 */
	@CustomFunctionResolver("compareRiskCustomFunctionResolver0")
	public static boolean compareRisk(List<Double> portfolio, List<Double> limits) {
		if (portfolio == null || limits == null || portfolio.size() != limits.size())
			return false;
		for (int i = 0; i < portfolio.size(); i++){
			if (portfolio.get(i) > limits.get(i))
				return false;
		}
		return true;
	}
	
	@CustomFunctionResolver("changeLimitsResolver")
	public static List<?> changeLimits(List<Double> oldLimits, List<Double> newLimits){
		if (oldLimits == null && newLimits == null)
			return new ArrayList<Double>();
		else if (oldLimits == null && newLimits != null)
			return newLimits;
		else if (newLimits == null && oldLimits != null)
			return oldLimits;
		if (oldLimits.size() != newLimits.size())
			return oldLimits;
		return newLimits;
	}

	/**
	 * A StreamBase Custom Function Resolver Function. This method is used by 
	 * StreamBase for type checking.  Use this method
	 * to resolve functions that take lists or tuples as arguments 
	 * or return lists or tuples.  This method should be used to ensure the 
	 * data types of your function are correct.
	
	 * For custom functions that return simple types (the Java equivalents of the simple 
	 * StreamBase types: int, long, double, bool, string, blob, timestamp), the returned CompleteDataType
	 * must be the appropriate corresponding simple type (e.g., as returned by CompleteDataType.forInt()).
	 * For custom functions that return non-simple types (the Java equivalents of the non-simple
	 * StreamBase types: tuple, list), the returned CompleteDataType must fully describe the appropriate
	 * non-simple type (as returned by CompleteDataType.forTuple(Schema) or CompleteDataType.forList(CompleteDataType))
	 *{@see com.streambase.sb.client.CustomFunctionResolver}  
	 */
	public static CompleteDataType compareRiskCustomFunctionResolver0(
			CompleteDataType portfolio, CompleteDataType limits) {
		if (portfolio.getElementType() == CompleteDataType.forDouble() && limits.getElementType() == CompleteDataType.forDouble())
			return CompleteDataType.forBool();
		return null;
	}
	
	public static CompleteDataType changeLimitsResolver(
			CompleteDataType oldLimits, CompleteDataType newLimits) {
		if (oldLimits.getElementType() == CompleteDataType.forDouble() && newLimits.getElementType() == CompleteDataType.forDouble())
			return CompleteDataType.forList(CompleteDataType.forDouble());
		return null;
	}

}
